package org.apache.maven.archiva.configuration;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.Predicate;
import org.apache.commons.configuration.CombinedConfiguration;
import org.apache.maven.archiva.configuration.functors.FiletypeSelectionPredicate;
import org.apache.maven.archiva.configuration.io.registry.ConfigurationRegistryReader;
import org.codehaus.plexus.logging.AbstractLogEnabled;
import org.codehaus.plexus.personality.plexus.lifecycle.phase.Initializable;
import org.codehaus.plexus.personality.plexus.lifecycle.phase.InitializationException;
import org.codehaus.plexus.registry.RegistryException;
import org.codehaus.plexus.registry.commons.CommonsConfigurationRegistry;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * FileTypes 
 *
 * @author <a href="mailto:joakime@apache.org">Joakim Erdfelt</a>
 * @version $Id: FileTypes.java 589831 2007-10-29 20:03:58Z joakime $
 * 
 * @plexus.component role="org.apache.maven.archiva.configuration.FileTypes"
 */
public class FileTypes
    extends AbstractLogEnabled
    implements Initializable
{
    public static final String ARTIFACTS = "artifacts";

    public static final String AUTO_REMOVE = "auto-remove";

    public static final String INDEXABLE_CONTENT = "indexable-content";

    public static final String IGNORED = "ignored";

    /**
     * @plexus.requirement
     */
    private ArchivaConfiguration archivaConfiguration;

    /**
     * Map of default values for the file types.
     */
    private Map<String, List<String>> defaultTypeMap = new HashMap<String, List<String>>();

    /**
     * <p>
     * Get the list of patterns for a specified filetype.
     * </p>
     * 
     * <p>
     * You will always get a list.  In this order.
     *   <ul>
     *     <li>The Configured List</li>
     *     <li>The Default List</li>
     *     <li>A single item list of <code>"**<span>/</span>*"</code></li>
     *   </ul>
     * </p>
     * 
     * @param id the id to lookup.
     * @return the list of patterns.
     */
    public List<String> getFileTypePatterns( String id )
    {
        Configuration config = archivaConfiguration.getConfiguration();
        Predicate selectedFiletype = new FiletypeSelectionPredicate( id );
        FileType filetype = (FileType) CollectionUtils.find( config.getRepositoryScanning().getFileTypes(),
                                                             selectedFiletype );

        if ( ( filetype != null ) && CollectionUtils.isNotEmpty( filetype.getPatterns() ) )
        {
            return filetype.getPatterns();
        }

        List<String> defaultPatterns = defaultTypeMap.get( id );

        if ( CollectionUtils.isEmpty( defaultPatterns ) )
        {
            return Collections.singletonList( "**/*" );
        }

        return defaultPatterns;
    }

    public void initialize()
        throws InitializationException
    {
        /* Initialize Default Type Map */
        defaultTypeMap.clear();

        String errMsg = "Unable to load default archiva configuration for FileTypes: ";
        
        try
        {
            CommonsConfigurationRegistry commonsRegistry = new CommonsConfigurationRegistry();

            // Configure commonsRegistry
            Field fld = commonsRegistry.getClass().getDeclaredField( "configuration" );
            fld.setAccessible( true );
            fld.set( commonsRegistry, new CombinedConfiguration() );
            commonsRegistry.enableLogging( getLogger() );
            commonsRegistry.addConfigurationFromResource( "org/apache/maven/archiva/configuration/default-archiva.xml" );
            
            // Read configuration as it was intended.
            ConfigurationRegistryReader configReader = new ConfigurationRegistryReader();
            Configuration defaultConfig = configReader.read( commonsRegistry );

            // Store the default file type declaration.
            List<FileType> filetypes = defaultConfig.getRepositoryScanning().getFileTypes();
            for ( FileType filetype : filetypes )
            {
                List<String> patterns = defaultTypeMap.get( filetype.getId() );
                if ( patterns == null )
                {
                    patterns = new ArrayList<String>();
                }
                patterns.addAll( filetype.getPatterns() );

                defaultTypeMap.put( filetype.getId(), patterns );
            }
        }
        catch ( RegistryException e )
        {
            throw new InitializationException( errMsg + e.getMessage(), e );
        }
        catch ( SecurityException e )
        {
            throw new InitializationException( errMsg + e.getMessage(), e );
        }
        catch ( NoSuchFieldException e )
        {
            throw new InitializationException( errMsg + e.getMessage(), e );
        }
        catch ( IllegalArgumentException e )
        {
            throw new InitializationException( errMsg + e.getMessage(), e );
        }
        catch ( IllegalAccessException e )
        {
            throw new InitializationException( errMsg + e.getMessage(), e );
        }
    }
}
